
#include "simp/SimpSolver.h"
#include "api/ExtendedAPI.h"
#include "api/ipasirglucose40glue.cc"

using namespace std;
using namespace Glucose;

class IPAsirGlucoseExtended : public IPAsirGlucose {
  typedef IPAsirGlucose base;
public:
  IPAsirGlucoseExtended () {
  }
  void add_clauses(int * clauses, int length){
	 for (int i=0; i<length; i++) {
		base::add(clauses[i]);
	 }
  }
  int * model(){
	 int vars = nVars();
	 int * ansModel = (int *)malloc(sizeof(int)*vars);
	 for(int i=0; i<vars; i++){
		ansModel[i] = val(i+1);
	 }
	 return ansModel;
  }
  void add_blocking_clause(int * literals, int length){
	 // all variable
	 if (literals == NULL){
		int * ansModel = model();
		for(int i=0; i<nVars(); i++)
		  base::add(-ansModel[i]);
		base::add(0);
	 }
	 else{
		for(int i=0; i<length; i++){
		  base::add(-literals[i]);
		}
		base::add(0);
	 }
  }
  void release(int * p){
	 free(p);
  }
  double gettime(){
	 return getime(); 
  }
  void set_frozen(int literal){
	 // for only "simp" version
	 setFrozen(literal, true);
  }
  int is_eliminated(int literal){
	 // for only "simp" version
	 return isEliminated(literal) ? 1 : 0;
  }
};

static IPAsirGlucoseExtended * importex (void * s) { return (IPAsirGlucoseExtended*) s; }
void isat_set_verbosity(void * s, int v){ importex(s) -> verbosity = v; }
int isat_vars(void * s){ return importex(s) -> nVars(); }
void isat_add_clauses(void * s, int * clauses, int length){ importex(s) -> add_clauses(clauses,length);}
int * isat_model(void * s){ return importex(s) -> model();}
void isat_add_blocking_clause(void * s, int * literals, int length){ importex(s) -> add_blocking_clause(literals,length);}
void isat_clean(void * s, int * p){ importex(s) -> release(p);}
double isat_get_time(void * s){ return importex(s) -> gettime();}
// void isat_add_PB(void * s, int * coefficients, int * variables);
// int * isat_MUS(void * s);
void isat_freeze(void * s, int literal){ importex(s) -> set_frozen(literal); }
int isat_is_eliminated(void * s, int literal){ return importex(s) -> is_eliminated(literal);}
// void ipasir_delete_learnts(void * s);
// void ipasir_set_polarity(void * s, int * polarity);
// int * ipasir_polarity(void * s);
